%%Setup
clear all;
T=20;
A0 =100;
A1 = .05;
d = .20;
mu0 = .0025;
beta = .95;
Q0 =1000;
tau0 = .0;
%gg computes the tax revenues for the government given Lagrangian
%multiplier mu
gg = @(mu) ComputeG(A0,A1,d,Q0,tau0,beta,mu);
%Solve the Ramsey problem and associated government revenue for mu0
[G0 A B F P] = gg(mu0);
%Compute the Optimal u0
P21 = P(4,1:3); P22 = P(4,4);
z0 = [1;Q0;tau0];
u0 = -P22^(-1)*P21*z0;
%Set up variables to compute time series note t=1 refers to time 0.
y = zeros(4,T);
%uhat and tauhat are what the planner would choose if he could reset at time t,
%uhatdif and tauhatdif are the difference between those and what the
%planner is constrained to choose.
uhat = zeros(1,T); uhatdif = zeros(1,T);
tauhat = zeros(1,T); tauhatdif = zeros(1,T-1);
uhatdif(1) = 0;
uhat(1) = u0;
y(:,1) = [z0;u0];
%mu is the Lagrange multiplier associated with the constraint at time t.
mu = zeros(1,T);
mu(1) = mu0;
G = zeros(1,T);
G(1) = G0;
for t= 2:T
    %Iterate government policy
    y(:,t) = (A-B*F)*y(:,t-1);
    %update G
    G(t) = ( G(t-1) - beta*y(2,t)*y(3,t) )/beta;
    %Compute the mu if the government were able to reset its plan
    %ff is the tax revenues the government would receive if they reset the
    %plan with Lagrange multiplier mu minus current G
    ff = @(mu) gg(mu)-G(t);
    %find ff = 0
    mu(t) = fsolve(ff,mu(t-1));
    [temp Atemp Btemp Ftemp Ptemp] = gg(mu(t));
    %Compute alternative decisions
    P21temp = Ptemp(4,1:3); P22temp = P(4,4);
    uhat(t) =-P22temp^(-1)*P21temp*y(1:3,t);
    %tauhat(t) = -F*[y(1:3,t-1);uhat(t-1)];
    yhat =(Atemp-Btemp*Ftemp)*[y(1:3,t-1);uhat(t-1)];
    tauhat(t) = yhat(3);
    tauhatdif(t-1) = tauhat(t)-y(3,t);
    uhatdif(t) = uhat(t)-y(4,t);
end

%Plots.  Note tt is used to make the plot time index correct.
close all
figure(1)
plot(y(3,:));title('taxes')
disp('1 Q tau u')
y'
disp('-F')
-F


tt = [0:1:T-1];
figure(2)
subplot(2,2,1),plot(tt',y(2,:)'),ylabel('Q')
subplot(2,2,2),plot(tt',y(3,:)'),ylabel('\tau')
subplot(2,2,3),plot(tt',y(4,:)'),ylabel('u')


print -depsc ES_plot_1

figure(3)
plot(tt',y(2,:)')

figure(4)
plot(tt',uhatdif,'o');

figure(5)
tt2 =  (0:T-2)';
plot(tt2,tauhatdif,'o');

figure(6)
plot(tt',mu,'o')
figure(7)
plot(tt,G,'o')